/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mappings are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2004 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package jorm.rdb;

// Import clauses required by jorm
import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PAccessorGen;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PBindingCtrl;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PClassMappingCtrl;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PExceptionNoDSI;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PMappingCallback;
import org.objectweb.jorm.api.PMappingStructuresManager;
import org.objectweb.jorm.api.PNameIterator;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.genclass.api.FieldDesc;
import org.objectweb.jorm.genclass.api.GenClassProp;
import org.objectweb.jorm.lib.EmptyPNameIterator;
import org.objectweb.jorm.lib.MultiplePNameIterator;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.naming.api.PNameManager;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.eval.prefetch.api.PrefetchCache;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.LoggerFactory;
//import for the rdb mapper
import org.objectweb.jorm.api.PPrefetchTupleCollection;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapter;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapterException;
import org.objectweb.jorm.mapper.rdb.adapter.api.JoinedTable;
import org.objectweb.jorm.mapper.rdb.lib.RdbExtentGenInfos;
import org.objectweb.jorm.mapper.rdb.lib.RdbTupleCollection;
import org.objectweb.jorm.mapper.rdb.lib.PClassMappingDecoder;
import org.objectweb.jorm.mapper.rdb.lib.PMapperRdb;
import org.objectweb.jorm.mapper.rdb.lib.RdbPMappingStructuresManager;
import org.objectweb.jorm.mapper.rdb.lib.RdbConnectionWrapper;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbClassMapping;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbMapping;
import org.objectweb.jorm.mapper.rdb.lib.RdbPPolymorphicClass;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.eval.lib.MedorEvaluator;
import org.objectweb.medor.tuple.api.TupleCollection;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.eval.api.ConnectionResources;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.CalculatedParameterOperand;
import org.objectweb.medor.expression.lib.BasicParameterOperand;
import org.objectweb.medor.expression.lib.BasicOperand;
import org.objectweb.medor.query.rdb.lib.BasicRdbStringQueryLeaf;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.query.lib.SelectProject;
import org.objectweb.medor.datasource.api.DataStore;
import org.objectweb.medor.datasource.lib.ConnectionFactoryDataStore;
import org.objectweb.medor.expression.lib.BasicBinaryArithmeticOperator;
import org.objectweb.medor.expression.lib.Plus;
import org.objectweb.medor.eval.prefetch.api.PrefetchBuffer;
import org.objectweb.medor.eval.prefetch.api.IntermediaryPrefetchBuffer;
import org.objectweb.medor.eval.prefetch.lib.IntermediaryPrefetchBufferImpl;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.filter.jorm.lib.EncodePName;
import org.objectweb.medor.filter.jorm.lib.CompositePName;
import org.objectweb.medor.filter.jorm.lib.SinglePName;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.optim.lib.IndexesGenerator;
import org.objectweb.perseus.persistence.api.ConnectionHolder;
import org.objectweb.perseus.persistence.api.WorkingSet;
import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Collections;
import java.util.Collection;
import java.util.ArrayList;
import java.util.List;
import java.util.NoSuchElementException;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.mapper.rdb.genclass.RdbFieldDesc;
import org.objectweb.jorm.mapper.rdb.genclass.RdbGenClassProp;
import org.objectweb.jorm.mapper.rdb.lib.RdbPrefetchablePCM;


// Import clauses required by the object Olstore_user
import jorm.Olstore_userAccessor;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;

public class Olstore_userMapping
	extends org.objectweb.jonas_ejb.container.jorm.RdbFactory
	implements PClassMapping, PClassMappingCtrl, Loggable
		, PClassMappingDecoder
		, RdbPrefetchablePCM
    {
	protected static final String MAPPER_NAME = "rdb";
	protected static final String PROJECT_NAME = "jonas";
	//Fields
	protected static final String address_NAME = "address";
	protected static final String emailAdd_NAME = "emailAdd";
	protected static final String fname_NAME = "fname";
	protected static final String friends_NAME = "friends";
	protected static final String jonasCMRFriend_User_NAME = "jonasCMRFriend_User";
	protected static final String lname_NAME = "lname";
	protected static final String orders_NAME = "orders";
	protected static final String passwd_NAME = "passwd";
	protected static final String phoneNum_NAME = "phoneNum";
	protected static final String purchased_NAME = "purchased";
	protected static final String role_NAME = "role";
	protected static final String username_NAME = "username";
	//Hidden fields
	protected static final String address_JONASAUTOPKFIELD_NAME = "address_JONASAUTOPKFIELD";

	private final static String[][] SUPER_TYPES =  {};

	/** Fully quailied name of the persitent class */
	protected final static String JORM_CLASS_NAME =
		"jorm.Olstore_user";
	/** The actual binder which does the binding action */
	private PBinder binder;
	private PNameCoder classPNC;

	/** The meta object of the current jorm class */
	private Class moc;
	/** The mapper linked to this PClassMapping */
	private PMapper mapper;
	public PNameCoder addressPnc;
	public PNameCoder friendsPnc;
	PClassMapping friendsGCM;
	public PNameCoder jonasCMRFriend_UserPnc;
	PClassMapping jonasCMRFriend_UserGCM;
	public PNameCoder ordersPnc;
	PClassMapping ordersGCM;
	public PNameCoder purchasedPnc;
	PClassMapping purchasedGCM;
	/** the PClassMapping of the sub type */
	private PClassMapping[] subPCMs = null;

	//variables for the resolve method
	private int beginIndex = 0;
	private QueryTree queryTree = null;
	private FieldOperand[] fieldOperands = null;
	private String[] compositeFieldNames = null;
 	/**
	 * The logger instance used by this XMapping instance
	 * and the XBinding instances
	 */
	public Logger logger;
	public LoggerFactory loggerFactory;

	public Olstore_userMapping() {
 	}
	public PNameCoder getAddressPNameCoder() {
	    return addressPnc;
	}
	public PNameCoder getFriendsPNameCoder() {
	    return friendsPnc;
	}
	public PNameCoder getJonasCMRFriend_UserPNameCoder() {
	    return jonasCMRFriend_UserPnc;
	}
	public PNameCoder getOrdersPNameCoder() {
	    return ordersPnc;
	}
	public PNameCoder getPurchasedPNameCoder() {
	    return purchasedPnc;
	}

	// IMPLEMENTATION OF METHODS FROM THE PClassMappingCtrl INTERFACE

    public void setPNameCoder(PNameCoder pnc)
            throws PException, UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
    }

    public void setPNameCoder(String fn, PNameCoder pnc)
            throws PException, UnsupportedOperationException {
		if (address_NAME.equals(fn)) {
			addressPnc = (PNamingContext) pnc;
			return;
		}
		if (friends_NAME.equals(fn)) {
			friendsPnc = (PNamingContext) pnc;
			return;
		}
		if (jonasCMRFriend_User_NAME.equals(fn)) {
			jonasCMRFriend_UserPnc = (PNamingContext) pnc;
			return;
		}
		if (orders_NAME.equals(fn)) {
			ordersPnc = (PNamingContext) pnc;
			return;
		}
		if (purchased_NAME.equals(fn)) {
			purchasedPnc = (PNamingContext) pnc;
			return;
		}
		throw new UnsupportedOperationException("This class does'nt contain field reference: " + fn);
    }

    public void setGenClassMapping(PClassMapping pcm)
            throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
    }

    public void setGenClassMapping(String fn, PClassMapping pcm)
            throws UnsupportedOperationException {
		if (friends_NAME.equals(fn)) {
			friendsGCM = pcm;
			return;
		}
		if (jonasCMRFriend_User_NAME.equals(fn)) {
			jonasCMRFriend_UserGCM = pcm;
			return;
		}
		if (orders_NAME.equals(fn)) {
			ordersGCM = pcm;
			return;
		}
		if (purchased_NAME.equals(fn)) {
			purchasedGCM = pcm;
			return;
		}
		throw new UnsupportedOperationException("This class does not contain a " + fn + " field referencing a generic class");
    }

    public void setProjectName(String projectname) {
    }


	// IMPLEMENTATION OF METHODS FROM THE PClassMapping INTERFACE

	public void init(PMappingCallback m, MetaObject metaclass) throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");
		mapper = m;
		moc = (Class) metaclass;
		//the getPType method inits the ptype if not already done
		PType ptype = mapper.getPTypeSpace().createPType(
				"jorm.Olstore_user",
				SUPER_TYPES);
		setPType(ptype);
        // PType Assignement to the PNC managing the reference address
        String[][] addressICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            "jorm.Olstore_address",
            addressICDef);
        addressPnc.setPType(ptype);
        
         try {
			adapter = ((PMapperRdb) mapper).getRdbAdapter();
			if (adapter == null) {
				throw new PException("No rdb adapter found on the mapper: " + mapper);
			}

         // Initialization of the gen class /jorm.Olstore_user/jonasCMRFriend_User
        FieldDesc[] fds = null;
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.Olstore_user_username", "", PTypeSpace.STRING, "olstore_user_username", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) jonasCMRFriend_UserGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) jonasCMRFriend_UserGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.Olstore_friend_id", "", PTypeSpace.OBJINT, "id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) jonasCMRFriend_UserGCM).setElemDesc(fds);
			((RdbGenClassProp) jonasCMRFriend_UserGCM).setTableName("olstore_friend");
			((RdbGenClassProp) jonasCMRFriend_UserGCM).setColocated(true);
			((RdbGenClassProp) jonasCMRFriend_UserGCM).setColocatedMaster(false);
			((RdbGenClassProp) jonasCMRFriend_UserGCM).setReadOnly(true);
			((RdbGenClassProp) jonasCMRFriend_UserGCM).setTypeConverter(adapter);
        String[] jonasCMRFriend_UserGCDef = {"java.util.Collection"};
        String[][] jonasCMRFriend_UserICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            jonasCMRFriend_UserGCDef,
            "jorm.Olstore_friend",
            jonasCMRFriend_UserICDef);
        ((GenClassProp) jonasCMRFriend_UserGCM).setPType(ptype);
        jonasCMRFriend_UserPnc.setPType(ptype);
        jonasCMRFriend_UserGCM.init(m, null);

         // Initialization of the gen class /jorm.Olstore_user/purchased
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.Olstore_user_username", "", PTypeSpace.STRING, "olstore_user_username", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) purchasedGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) purchasedGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.Olstore_item_itemId", "", PTypeSpace.OBJINT, "olstore_item_itemId", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) purchasedGCM).setElemDesc(fds);
			((RdbGenClassProp) purchasedGCM).setTableName("OLSTORE_USER_OLSTORE_ITEM");
			((RdbGenClassProp) purchasedGCM).setColocated(false);
			((RdbGenClassProp) purchasedGCM).setColocatedMaster(false);
			((RdbGenClassProp) purchasedGCM).setReadOnly(true);
			((RdbGenClassProp) purchasedGCM).setTypeConverter(adapter);
        String[] purchasedGCDef = {"java.util.Collection"};
        String[][] purchasedICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            purchasedGCDef,
            "jorm.Olstore_item",
            purchasedICDef);
        ((GenClassProp) purchasedGCM).setPType(ptype);
        purchasedPnc.setPType(ptype);
        purchasedGCM.init(m, null);

         // Initialization of the gen class /jorm.Olstore_user/orders
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.Olstore_user_username", "", PTypeSpace.STRING, "olstore_user_username", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) ordersGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) ordersGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.Olstore_order_orderId", "", PTypeSpace.OBJINT, "orderId", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) ordersGCM).setElemDesc(fds);
			((RdbGenClassProp) ordersGCM).setTableName("olstore_order");
			((RdbGenClassProp) ordersGCM).setColocated(true);
			((RdbGenClassProp) ordersGCM).setColocatedMaster(false);
			((RdbGenClassProp) ordersGCM).setReadOnly(true);
			((RdbGenClassProp) ordersGCM).setTypeConverter(adapter);
        String[] ordersGCDef = {"java.util.Collection"};
        String[][] ordersICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            ordersGCDef,
            "jorm.Olstore_order",
            ordersICDef);
        ((GenClassProp) ordersGCM).setPType(ptype);
        ordersPnc.setPType(ptype);
        ordersGCM.init(m, null);

         // Initialization of the gen class /jorm.Olstore_user/friends
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.Olstore_user_username", "", PTypeSpace.STRING, "olstore_user_username", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) friendsGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) friendsGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.Olstore_friend_id", "", PTypeSpace.OBJINT, "olstore_friend_id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) friendsGCM).setElemDesc(fds);
			((RdbGenClassProp) friendsGCM).setTableName("OLSTORE_USER_OLSTORE_FRIEND");
			((RdbGenClassProp) friendsGCM).setColocated(false);
			((RdbGenClassProp) friendsGCM).setColocatedMaster(false);
			((RdbGenClassProp) friendsGCM).setReadOnly(true);
			((RdbGenClassProp) friendsGCM).setTypeConverter(adapter);
        String[] friendsGCDef = {"java.util.Collection"};
        String[][] friendsICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            friendsGCDef,
            "jorm.Olstore_friend",
            friendsICDef);
        ((GenClassProp) friendsGCM).setPType(ptype);
        friendsPnc.setPType(ptype);
        friendsGCM.init(m, null);
 	//register the structures and dependencies
	init(mapper.getPMappingStructuresManager());
        } catch (Exception e) {
			if(e instanceof RdbAdapterException)
				throw new PException(e, "Column type computing error: ");
			else
				e.printStackTrace();
		} 
			computeReadQuery();
		try {
			computeExtentQuery();
		} catch (ExpressionException e) {
			throw new PException(e);
		}
		computePrefetchedExtentQuery();
		}

	public void init(PMappingStructuresManager pmsm) throws PException {
		pmsm.declareClass(JORM_CLASS_NAME);
		try {
			        RdbPMappingStructuresManager msm = (RdbPMappingStructuresManager) pmsm;
        if (mapper == null) {
	       	mapper = msm.getPMapper();
	    }
	    if (adapter == null) {
			adapter = ((PMapperRdb) mapper).getRdbAdapter();
			if (adapter == null) {
				throw new PException("No rdb adapter found on the mapper: " + mapper);
			}
		}
	    // Initialization of the structures of the class jorm.Olstore_user
			        	msm.addTableColumn(JORM_CLASS_NAME, "olstore_user", "lname", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "olstore_user", "fname", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "olstore_user", "emailAdd", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "olstore_user", "passwd", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "olstore_user", "username", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, -1, -1), true, true, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "olstore_user", "role", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "olstore_user", "phoneNum", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "olstore_user", "olstore_address_JPK_", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), false, -1, -1), false, false, true);
	    		        // Initialization of the structures of the gen class /jorm.Olstore_user/jonasCMRFriend_User
		        	msm.addTableColumn(JORM_CLASS_NAME, "olstore_friend", "olstore_user_username", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "olstore_friend", "id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.Olstore_user/purchased
		        	msm.addTableColumn(JORM_CLASS_NAME, "OLSTORE_USER_OLSTORE_ITEM", "olstore_user_username", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "OLSTORE_USER_OLSTORE_ITEM", "olstore_item_itemId", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.Olstore_user/orders
		        	msm.addTableColumn(JORM_CLASS_NAME, "olstore_order", "olstore_user_username", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "olstore_order", "orderId", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.Olstore_user/friends
		        	msm.addTableColumn(JORM_CLASS_NAME, "OLSTORE_USER_OLSTORE_FRIEND", "olstore_user_username", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "OLSTORE_USER_OLSTORE_FRIEND", "olstore_friend_id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
					//dependencies
	        pmsm.addDependency(JORM_CLASS_NAME, "jorm.Olstore_friend");
	        pmsm.addDependency(JORM_CLASS_NAME, "jorm.Olstore_order");
	        pmsm.addDependency(JORM_CLASS_NAME, "jorm.Olstore_address");
	        pmsm.addDependency(JORM_CLASS_NAME, "jorm.Olstore_item");
			} catch(Exception e) {
			throw new PException("Error while registering mapping structures: " + e.getMessage());
		}
		classDefined(pmsm);
	}
	
	protected void classDefined() throws PException {
		//register class in the cluster
        classDefined(mapper.getPMappingStructuresManager());
	}

	public void classDefined(PMappingStructuresManager pmsm) throws PException {
		if (pmsm == null) {
			throw new PException("PMappingStructureManager is null: " + JORM_CLASS_NAME + " cannot be registered in a cluster.");
		}
		//register class in the cluster
        pmsm.classDefined(JORM_CLASS_NAME);
	}
	
	public PMapper getPMapper() {
		return mapper;
	}

	public PBinder getPBinder() {
		return binder;
	}

	public PNameCoder getClassPNameCoder() {
		return classPNC;
	}

	/** This method is used only in the case of the current
	 * class is a gen class and contains references*/
	public PNameCoder getPNameCoder()
		throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	public PNameCoder getPNameCoder(String fn)
		throws UnsupportedOperationException {
			if (address_NAME.equals(fn))
				return addressPnc;
			if (friends_NAME.equals(fn))
				return friendsPnc;
			if (jonasCMRFriend_User_NAME.equals(fn))
				return jonasCMRFriend_UserPnc;
			if (orders_NAME.equals(fn))
				return ordersPnc;
			if (purchased_NAME.equals(fn))
				return purchasedPnc;
		throw new UnsupportedOperationException("This class does'nt contain field reference " + fn);
	}

	public String getClassName() {
		return JORM_CLASS_NAME;
	}

	/** This method is used only in the case of the current
	 * class is a gen class and contains another gen class*/
	public PClassMapping getGenClassMapping() throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	public PClassMapping getGenClassMapping(String fn)
		throws UnsupportedOperationException {
			if (friends_NAME.equals(fn))
				return friendsGCM;
			if (jonasCMRFriend_User_NAME.equals(fn))
				return jonasCMRFriend_UserGCM;
			if (orders_NAME.equals(fn))
				return ordersGCM;
			if (purchased_NAME.equals(fn))
				return purchasedGCM;
			throw new UnsupportedOperationException("This class does not contain a " + fn + " field which would reference a generic class");
	}

	public boolean isConform(String mapperName) {
		return mapperName.startsWith(MAPPER_NAME);
	}

	public String getProjectName() {
		return PROJECT_NAME;
	}

	public MetaObject getMetaInfo() {
		return moc;
	}

	public String[][] getSuperClassNames() {
		return SUPER_TYPES;
	}

	public void setPBinder(PBinder b) {
		binder = b;
	}

	public void setClassPNameCoder(PNameCoder pnc) throws PException {
		classPNC = pnc;
	}

	public PBinding createPBinding() throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");
		PBinding binding = new jorm.Olstore_userBinding();
		binding.init(this);
		return binding;
	}
	/** @deprecated */
    public Iterator getPNameIterator(Object conn) throws PException {
		return getPNameIterator(conn, false, false, null);
    }

	public PNameIterator getPNameIterator(Object conn,
	                                      boolean withSubType,
	                                      boolean prefetching,
	                                      Object txctx) throws PException {
		PNameIterator it = null;
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
		try {
			String mapperName = mapper.getMapperName();
			if(mapperName.indexOf(".") != -1)
				mapperName = mapperName.substring(0, mapperName.indexOf("."));	
			RdbMapping rdbm = (RdbMapping) mapper.getMetaInfoManager().getClass(this.getClassName()).getClassProject(this.getProjectName()).getMapping(mapperName);
			RdbClassMapping rdbcm = (RdbClassMapping) rdbm.getClassMapping();
			String queryToExecute = "";
			int beginIndex = 0;
			if(withSubType){
				//the list of fields belonging to the primary key
				String pk = "olstore_user.username";
				queryToExecute = rdbcm.getRdbInheritanceQuery().getExtentQuery(mapper,prefetching, true, pk, this);	
				//System.out.println("query to execute:" + queryToExecute + " compared with the generated query: " + (prefetching ? prefetchedExtentQuery : extentQuery));
				beginIndex =  rdbcm.getRdbInheritanceQuery().getExtentFieldsNames(prefetching, true).size() + 1;
			}
			else{
				queryToExecute = (prefetching ? prefetchedExtentQuery : extentQuery);
				//System.out.println("query to execute:" + queryToExecute);
				//compute the begining index for the PName: number of fields to select + 1
				beginIndex = 8 + 1;
			}
			PreparedStatement ps = connection.prepareStatement(queryToExecute);
			it = new Olstore_userResultIterator(txctx, ps, ps.executeQuery(), binder, prefetching, withSubType, logger, beginIndex, this);
        } catch (SQLException e) {
            throw new PExceptionIO(e, "SQL problem while computing the PName Iterator: class <Olstore_user>.");
        } catch (MedorException e) {
            throw new PExceptionIO(e, "Medor problem while initializing the PName Iterator: class <Olstore_user>.");
        }
				return it;
			}

	public PClassMapping[] getSubPCMs() throws PException {
		PType ptype = getPType();
		if (ptype == null) {
			throw new PException("No PType defined for the PClassMapping of the class" + JORM_CLASS_NAME);
		}
		PType[] subTypes = ptype.getSubTypes();
		if (subTypes == null || subTypes.length == 0) {
			//No direct subType are mapped then return null
			return null;
		}
		if (subPCMs == null || subPCMs.length != subTypes.length) {
			//compute the PCM array
			PMapper mapper = getPMapper();
			subPCMs = new PClassMapping[subTypes.length];
			for (int i=0; i<subTypes.length; i++) {
				subPCMs[i] = mapper.lookup(subTypes[i].getJormName());
			}
		}
		return subPCMs;
	}
	
	// ADD YO //
	public synchronized PName resolve(Object conn, PName pname) throws PException {
				//mapper rdb but not polymorphic
		return pname;
	 	}
	
	public boolean match(Object obj, boolean intermediaryTuple) throws PException {
					return false;
				}
	
	public PName getDecodedPName(TupleCollection tc, PName pname, boolean intermediaryTuple) throws PException {
		PBinder binder = null;
	//test if the tuple is an instance of this class
	if (this.match(tc, intermediaryTuple)) {
		binder  = this.getPBinder();
	} else {
		//test for all the children
		boolean match = false;
		for (int i = 0; i<subPCMs.length && !match; i++) {
				match = subPCMs[i].match(tc, intermediaryTuple);
				binder = subPCMs[i].getPBinder();
		}
		//if no pcm found, error
		if (!match)
			throw new PException("Problem with Medor while resolving pname on the PCM " + JORM_CLASS_NAME + ". No class found for the pname " + pname.toString() + ".");
	}
			return binder.decodeString(pname.encodeString());
		}
	// END ADD YO //
	
    public void configureRefFields(ReferenceConfigurator rc) throws PException, UnsupportedOperationException {
        PClassMapping gcm = null;
		addressPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, address_NAME, "jorm.Olstore_address");
        friendsPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, friends_NAME, new String[]{"java.util.Collection"});
        friendsGCM = rc.getGenClassMapping(JORM_CLASS_NAME, friends_NAME, new String[]{"java.util.Collection"}, "jorm.Olstore_friend");
        gcm = friendsGCM;
        jonasCMRFriend_UserPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, jonasCMRFriend_User_NAME, new String[]{"java.util.Collection"});
        jonasCMRFriend_UserGCM = rc.getGenClassMapping(JORM_CLASS_NAME, jonasCMRFriend_User_NAME, new String[]{"java.util.Collection"}, "jorm.Olstore_friend");
        gcm = jonasCMRFriend_UserGCM;
        ordersPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, orders_NAME, new String[]{"java.util.Collection"});
        ordersGCM = rc.getGenClassMapping(JORM_CLASS_NAME, orders_NAME, new String[]{"java.util.Collection"}, "jorm.Olstore_order");
        gcm = ordersGCM;
        purchasedPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, purchased_NAME, new String[]{"java.util.Collection"});
        purchasedGCM = rc.getGenClassMapping(JORM_CLASS_NAME, purchased_NAME, new String[]{"java.util.Collection"}, "jorm.Olstore_item");
        gcm = purchasedGCM;
    }

	public PType getPType() {
		return  binder.getPType();
	}

	public void setPType(PType pt) {
		binder.setPType(pt);
	}


	public void write(PBinding pb, Object conn, PAccessor pa)
		throws PException {
		Olstore_userAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof Olstore_userAccessor)
			paccessor = (Olstore_userAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte status = pb.getStatus();
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_WRITE);
		if (nextState == PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized WRITE operation, status=" + status);
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
		try {
			String sqlquery;
		    if (status == PBinding.LIFECYCLE_DELTOWRITE) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Remove");
//Table olstore_user : readOnly=false mustGenerateUpdate=true colocatedTable=false joins=$tab.joins
                sqlquery = deleteolstore_userTable(connection, conn, paccessorGen, paccessor, pb.getPName());
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [Olstore_user]: problematic SQL query -> {" + sqlquery + "}");
                }
            } else if (status==PBinding.LIFECYCLE_NEWTOWRITE) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Insert");
                sqlquery = insertolstore_userTable(connection, conn, paccessorGen, paccessor, pb.getPName());
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [Olstore_user]: problematic SQL query -> {" + sqlquery + "}");
                }
		    } else if (status==PBinding.LIFECYCLE_ACTIVEFORIO) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Update");
                sqlquery = updateolstore_userTable(connection, conn, paccessorGen, paccessor, pb.getPName());
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [Olstore_user]: problematic SQL query -> {" + sqlquery + "}");
                }
		    }
		} catch (java.sql.SQLException se) {
			throw new PExceptionIO(se, "SQL problem while writing the DSI.");
		} catch (java.io.IOException ioe) {
			throw new PExceptionIO(ioe, "Stream IO problem while writing the DSI.");
		} catch (RdbAdapterException rae) {
			throw new PExceptionIO(rae, "SQL problem while writing the DSI.");
		}
		// Changing the current state
		((PBindingCtrl) pb).setStatus(nextState);
	}

	public void read(PBinding pb, Object conn, PAccessor pa) throws PException {
		if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Read");
		byte status = pb.getStatus();
		Olstore_userAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof Olstore_userAccessor)
			paccessor = (Olstore_userAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
		PName pname = pb.getPName();
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
        PreparedStatement pstmt = null;
        ResultSet resultSet = null;
        PException resex = null;
		try {
            pstmt = connection.prepareStatement(readQuery);
            adapter.setString(pstmt, 1, pname.encodeString());
            resultSet = pstmt.executeQuery();
            if(resultSet.next() == false)
                throw new PExceptionNoDSI(
                    "No entry matching with the following identifier: " + pname);
            PName refname;
            Object fieldval;
            long notnullval;
			if (paccessor != null) {
				//read public primitive fields
                paccessor.paSetEmailAdd(adapter.getString(resultSet, 1, null));
                paccessor.paSetFname(adapter.getString(resultSet, 2, null));
                paccessor.paSetLname(adapter.getString(resultSet, 3, null));
                paccessor.paSetPasswd(adapter.getString(resultSet, 5, null));
                paccessor.paSetPhoneNum(adapter.getString(resultSet, 6, null));
                paccessor.paSetRole(adapter.getString(resultSet, 7, null));
                paccessor.paSetUsername(adapter.getString(resultSet, 8, null));
				//decode PName for references
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@17a906e / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1a05bdb / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@d83365 / fieldName=jonasCMRFriend_User / join=null
				//fieldName=username
                fieldval = adapter.getString(resultSet, 8, null);
                if (resultSet.wasNull()) {
                    refname = jonasCMRFriend_UserPnc.decodeString(null);
                } else {
                    refname = jonasCMRFriend_UserPnc.decodeString((java.lang.String) fieldval);
                }
                paccessor.paSetJonasCMRFriend_User(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@eebf17 / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1a05bdb / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@171b4ca / fieldName=purchased / join=null
				//fieldName=username
                fieldval = adapter.getString(resultSet, 8, null);
                if (resultSet.wasNull()) {
                    refname = purchasedPnc.decodeString(null);
                } else {
                    refname = purchasedPnc.decodeString((java.lang.String) fieldval);
                }
                paccessor.paSetPurchased(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@439d3b / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@10794d4 / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@6f4652 / fieldName=address / join=null
				//fieldName=address_JONASAUTOPKFIELD
                fieldval = adapter.getOint(resultSet, 4, null);
                if (resultSet.wasNull()) {
                    refname = addressPnc.decodeOint(null);
                } else {
                    refname = addressPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessor.paSetAddress(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@bc0eba / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1a05bdb / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@d21538 / fieldName=orders / join=null
				//fieldName=username
                fieldval = adapter.getString(resultSet, 8, null);
                if (resultSet.wasNull()) {
                    refname = ordersPnc.decodeString(null);
                } else {
                    refname = ordersPnc.decodeString((java.lang.String) fieldval);
                }
                paccessor.paSetOrders(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@6af790 / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1a05bdb / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@164804 / fieldName=friends / join=null
				//fieldName=username
                fieldval = adapter.getString(resultSet, 8, null);
                if (resultSet.wasNull()) {
                    refname = friendsPnc.decodeString(null);
                } else {
                    refname = friendsPnc.decodeString((java.lang.String) fieldval);
                }
                paccessor.paSetFriends(refname, conn);
			} else if (paccessorGen != null) {
				//read public primitive fields
                paccessorGen.paSetStringField(emailAdd_NAME, adapter.getString(resultSet, 1, null));
                paccessorGen.paSetStringField(fname_NAME, adapter.getString(resultSet, 2, null));
                paccessorGen.paSetStringField(lname_NAME, adapter.getString(resultSet, 3, null));
                paccessorGen.paSetStringField(passwd_NAME, adapter.getString(resultSet, 5, null));
                paccessorGen.paSetStringField(phoneNum_NAME, adapter.getString(resultSet, 6, null));
                paccessorGen.paSetStringField(role_NAME, adapter.getString(resultSet, 7, null));
                paccessorGen.paSetStringField(username_NAME, adapter.getString(resultSet, 8, null));
				//decode PName for references
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@6af790 / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1a05bdb / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@164804 / fieldName=friends / join=null
				//fieldName=username
                fieldval = adapter.getString(resultSet, 8, null);
                if (resultSet.wasNull()) {
                    refname = friendsPnc.decodeString(null);
                } else {
                    refname = friendsPnc.decodeString((java.lang.String) fieldval);
                }
                paccessorGen.paSetRefField(friends_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@6af790 / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1a05bdb / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@164804 / fieldName=friends / join=null
				//fieldName=username
                fieldval = adapter.getString(resultSet, 8, null);
                if (resultSet.wasNull()) {
                    refname = friendsPnc.decodeString(null);
                } else {
                    refname = friendsPnc.decodeString((java.lang.String) fieldval);
                }
                paccessorGen.paSetRefField(friends_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@6af790 / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1a05bdb / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@164804 / fieldName=friends / join=null
				//fieldName=username
                fieldval = adapter.getString(resultSet, 8, null);
                if (resultSet.wasNull()) {
                    refname = friendsPnc.decodeString(null);
                } else {
                    refname = friendsPnc.decodeString((java.lang.String) fieldval);
                }
                paccessorGen.paSetRefField(friends_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@6af790 / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1a05bdb / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@164804 / fieldName=friends / join=null
				//fieldName=username
                fieldval = adapter.getString(resultSet, 8, null);
                if (resultSet.wasNull()) {
                    refname = friendsPnc.decodeString(null);
                } else {
                    refname = friendsPnc.decodeString((java.lang.String) fieldval);
                }
                paccessorGen.paSetRefField(friends_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@6af790 / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1a05bdb / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@164804 / fieldName=friends / join=null
				//fieldName=username
                fieldval = adapter.getString(resultSet, 8, null);
                if (resultSet.wasNull()) {
                    refname = friendsPnc.decodeString(null);
                } else {
                    refname = friendsPnc.decodeString((java.lang.String) fieldval);
                }
                paccessorGen.paSetRefField(friends_NAME, refname, conn);
			}
		} catch (PException pe) {
			resex = pe;
		} catch (Exception pe) {
			resex = new PExceptionIO(pe, "Problem while reading the DSI.");
		} finally {
		    try {
		        if (resultSet != null) {
			        resultSet.close();
			        resultSet = null;
			    }
			    if (pstmt != null) {
			        pstmt.close();
			        pstmt = null;
			    }
			} catch (SQLException se) {
				if (resex == null) {
			    	resex = new PExceptionIO(se, "Problem while closing ResultSet or Statment");
			    }
			}
			if (resex != null) {
				throw resex;
			}
		}
		((PBindingCtrl) pb).setStatus(nextState);
	}

	public void read(PBinding pb, Object conn, PAccessor pa, Object txctx) throws PException {
	    PrefetchCache pc = getPMapper().getPrefetchCache();
	    if (pc == null || txctx == null) {
	        read(pb, conn, pa);
	        return;
	    }
		PName pname = pb.getPName();
	    try {
	        Tuple tuple = pc.getPrefetchTuple(pname, this, txctx);
	        if (tuple == null) {
	            read(pb, conn, pa);
	            return;
	        }
		    Olstore_userAccessor paccessor = null;
		    PAccessorGen paccessorGen = null;
		    if (pa == null)
			    throw new PExceptionProtocol("Accessor must not be null");
		    else if (pa instanceof Olstore_userAccessor)
			    paccessor = (Olstore_userAccessor) pa;
		    else if (pa instanceof PAccessorGen)
			    paccessorGen = (PAccessorGen) pa;
		    else
			    throw new PExceptionProtocol("Wrong type of accessor: " + pa);
			byte status = pb.getStatus();
		    byte nextState = PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		    if (nextState==PBinding.LIFECYCLE_ERROR)
			    throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
            Object fval;
            long notnullval;
            if (paccessor != null) {
				fval =
					(new java.lang.Integer(tuple.getInt(4)));
                if (tuple.isDefined(4)) {
                    paccessor.paSetAddress(addressPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessor.paSetAddress(addressPnc.getNull(), conn);
                }
                paccessor.paSetEmailAdd(tuple.getString(1));
                paccessor.paSetFname(tuple.getString(2));
				fval =
					tuple.getString(8);
                if (tuple.isDefined(8)) {
                    paccessor.paSetFriends(friendsPnc.decodeString((java.lang.String) fval), conn);
                } else {
                    paccessor.paSetFriends(friendsPnc.getNull(), conn);
                }
				fval =
					tuple.getString(8);
                if (tuple.isDefined(8)) {
                    paccessor.paSetJonasCMRFriend_User(jonasCMRFriend_UserPnc.decodeString((java.lang.String) fval), conn);
                } else {
                    paccessor.paSetJonasCMRFriend_User(jonasCMRFriend_UserPnc.getNull(), conn);
                }
                paccessor.paSetLname(tuple.getString(3));
				fval =
					tuple.getString(8);
                if (tuple.isDefined(8)) {
                    paccessor.paSetOrders(ordersPnc.decodeString((java.lang.String) fval), conn);
                } else {
                    paccessor.paSetOrders(ordersPnc.getNull(), conn);
                }
                paccessor.paSetPasswd(tuple.getString(5));
                paccessor.paSetPhoneNum(tuple.getString(6));
				fval =
					tuple.getString(8);
                if (tuple.isDefined(8)) {
                    paccessor.paSetPurchased(purchasedPnc.decodeString((java.lang.String) fval), conn);
                } else {
                    paccessor.paSetPurchased(purchasedPnc.getNull(), conn);
                }
                paccessor.paSetRole(tuple.getString(7));
                paccessor.paSetUsername(tuple.getString(8));
            } else if (paccessorGen != null) {
				fval =
					(new java.lang.Integer(tuple.getInt(4)));
                if (tuple.isDefined(4)) {
                    paccessorGen.paSetRefField(address_NAME, addressPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessorGen.paSetRefField(address_NAME, addressPnc.getNull(), conn);
                }
                paccessorGen.paSetStringField(emailAdd_NAME, tuple.getString(1));
                paccessorGen.paSetStringField(fname_NAME, tuple.getString(2));
				fval =
					tuple.getString(8);
                if (tuple.isDefined(8)) {
                    paccessorGen.paSetRefField(friends_NAME, friendsPnc.decodeString((java.lang.String) fval), conn);
                } else {
                    paccessorGen.paSetRefField(friends_NAME, friendsPnc.getNull(), conn);
                }
				fval =
					tuple.getString(8);
                if (tuple.isDefined(8)) {
                    paccessorGen.paSetRefField(jonasCMRFriend_User_NAME, jonasCMRFriend_UserPnc.decodeString((java.lang.String) fval), conn);
                } else {
                    paccessorGen.paSetRefField(jonasCMRFriend_User_NAME, jonasCMRFriend_UserPnc.getNull(), conn);
                }
                paccessorGen.paSetStringField(lname_NAME, tuple.getString(3));
				fval =
					tuple.getString(8);
                if (tuple.isDefined(8)) {
                    paccessorGen.paSetRefField(orders_NAME, ordersPnc.decodeString((java.lang.String) fval), conn);
                } else {
                    paccessorGen.paSetRefField(orders_NAME, ordersPnc.getNull(), conn);
                }
                paccessorGen.paSetStringField(passwd_NAME, tuple.getString(5));
                paccessorGen.paSetStringField(phoneNum_NAME, tuple.getString(6));
				fval =
					tuple.getString(8);
                if (tuple.isDefined(8)) {
                    paccessorGen.paSetRefField(purchased_NAME, purchasedPnc.decodeString((java.lang.String) fval), conn);
                } else {
                    paccessorGen.paSetRefField(purchased_NAME, purchasedPnc.getNull(), conn);
                }
                paccessorGen.paSetStringField(role_NAME, tuple.getString(7));
                paccessorGen.paSetStringField(username_NAME, tuple.getString(8));
            }
		((PBindingCtrl) pb).setStatus(nextState);
		} catch (MedorException me) {
		    throw new PExceptionProtocol(me, "Problem while trying to prefetch data from Medor");
		}
    }

	public boolean exist(PBinding pb, Object conn) throws PException {
		if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Exist");
		byte status = pb.getStatus();
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_EXIST);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized EXIST operation, status=" + status);
		((PBindingCtrl) pb).setStatus(nextState);
		PName pname = pb.getPName();
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        PException resex = null;
        boolean exist = false;
		try {
            pstmt = connection.prepareStatement("SELECT olstore_user.username FROM olstore_user WHERE olstore_user.username = ?");
            adapter.setString(pstmt, 1, pname.encodeString());
			rs = pstmt.executeQuery();
            exist = rs.next();
		} catch (java.sql.SQLException se) {
			resex = new PExceptionIO(se, "SQL problem while testing the existence of a DSI.");
		} finally {
		    try {
			    if (rs != null) {
			        rs.close();
			        rs = null;
			    }
			    if (pstmt != null) {
			        pstmt.close();
			        pstmt = null;
			    }
			} catch (SQLException se) {
				if (resex == null) {
					resex = new PExceptionIO(se, "Problem while closing ResultSet or Statment");
			    }
			}
			if (resex != null) {
				throw resex;
			}
		}
		return exist;
	}

	//the map of [pclassmapping->table of int]
	//build dynamically
	private HashMap associationTable =  new HashMap();
	
	public HashMap getAssociationTable(){
		return associationTable;
	}
	
	public void addAssociation(PClassMapping targetClass, int[] indexes){
		associationTable.put(targetClass, indexes);
	}
	
	public int[] getIndexesTable(PClassMapping targetClass){
		return (int[]) associationTable.get(targetClass);
	}
	

    private String updateolstore_userTable(
        Connection connection,
        Object conn,
        PAccessorGen paccessorGen,
        Olstore_userAccessor paccessor,
    	PName pname)
        throws PException, SQLException, IOException, RdbAdapterException {
		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            if (paccessor != null) {
            pstmt = connection.prepareStatement(res = "UPDATE olstore_user SET lname = ?, fname = ?, emailAdd = ?, passwd = ?, username = ?, role = ?, phoneNum = ?, olstore_address_JPK_ = ? WHERE username = ?");
				//lname
                nullField = paccessor.paGetLname();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//fname
                nullField = paccessor.paGetFname();
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//emailAdd
                nullField = paccessor.paGetEmailAdd();
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 3, ((java.lang.String) nullField));
                }
				//passwd
                nullField = paccessor.paGetPasswd();
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 4, ((java.lang.String) nullField));
                }
				//username
                nullField = paccessor.paGetUsername();
                if (nullField == null) {
                    adapter.setNull(pstmt, 5, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 5, ((java.lang.String) nullField));
                }
				//role
                nullField = paccessor.paGetRole();
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 6, ((java.lang.String) nullField));
                }
				//phoneNum
                nullField = paccessor.paGetPhoneNum();
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 7, ((java.lang.String) nullField));
                }
				//olstore_address_JPK_
                nullField = addressPnc.encodeOint(paccessor.paGetAddress(conn));
                if (nullField == null) {
                    adapter.setNull(pstmt, 8, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 8, ((java.lang.Integer) nullField));
                }
				//username
                nullField = paccessor.paGetUsername();
                if (nullField == null) {
                    adapter.setNull(pstmt, 9, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 9, ((java.lang.String) nullField));
                }
            } else if (paccessorGen != null) {
            pstmt = connection.prepareStatement(res = "UPDATE olstore_user SET lname = ?, fname = ?, emailAdd = ?, passwd = ?, username = ?, role = ?, phoneNum = ?, olstore_address_JPK_ = ? WHERE username = ?");
				//lname
                nullField = paccessorGen.paGetStringField(lname_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//fname
                nullField = paccessorGen.paGetStringField(fname_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//emailAdd
                nullField = paccessorGen.paGetStringField(emailAdd_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 3, ((java.lang.String) nullField));
                }
				//passwd
                nullField = paccessorGen.paGetStringField(passwd_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 4, ((java.lang.String) nullField));
                }
				//username
                nullField = paccessorGen.paGetStringField(username_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 5, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 5, ((java.lang.String) nullField));
                }
				//role
                nullField = paccessorGen.paGetStringField(role_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 6, ((java.lang.String) nullField));
                }
				//phoneNum
                nullField = paccessorGen.paGetStringField(phoneNum_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 7, ((java.lang.String) nullField));
                }
				//olstore_address_JPK_
                nullField = addressPnc.encodeOint(paccessorGen.paGetRefField(address_NAME, conn));
                if (nullField == null) {
                    adapter.setNull(pstmt, 8, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 8, ((java.lang.Integer) nullField));
                }
				//username
                nullField = paccessor.paGetUsername();
                if (nullField == null) {
                    adapter.setNull(pstmt, 9, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 9, ((java.lang.String) nullField));
                }
            }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
    private String insertolstore_userTable(Connection connection,
                                                Object conn,
                                                PAccessorGen paccessorGen,
                                                Olstore_userAccessor paccessor,
    											PName pname)
        throws PException, SQLException, IOException, RdbAdapterException {

		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            if (paccessor != null) {
                pstmt = connection.prepareStatement(res = "INSERT INTO olstore_user (lname, fname, emailAdd, passwd, username, role, phoneNum, olstore_address_JPK_) VALUES (?, ?, ?, ?, ?, ?, ?, ?)");
				//lname
                nullField = paccessor.paGetLname();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//fname
                nullField = paccessor.paGetFname();
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//emailAdd
                nullField = paccessor.paGetEmailAdd();
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 3, ((java.lang.String) nullField));
                }
				//passwd
                nullField = paccessor.paGetPasswd();
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 4, ((java.lang.String) nullField));
                }
				//username
                nullField = paccessor.paGetUsername();
                if (nullField == null) {
                    adapter.setNull(pstmt, 5, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 5, ((java.lang.String) nullField));
                }
				//role
                nullField = paccessor.paGetRole();
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 6, ((java.lang.String) nullField));
                }
				//phoneNum
                nullField = paccessor.paGetPhoneNum();
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 7, ((java.lang.String) nullField));
                }
				//olstore_address_JPK_
                nullField = addressPnc.encodeOint(paccessor.paGetAddress(conn));
                if (nullField == null) {
                    adapter.setNull(pstmt, 8, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 8, ((java.lang.Integer) nullField));
                }
            } else if (paccessorGen != null) {
                pstmt = connection.prepareStatement(res = "INSERT INTO olstore_user (lname, fname, emailAdd, passwd, username, role, phoneNum, olstore_address_JPK_) VALUES (?, ?, ?, ?, ?, ?, ?, ?)");
				//lname
                nullField = paccessorGen.paGetStringField(lname_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//fname
                nullField = paccessorGen.paGetStringField(fname_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//emailAdd
                nullField = paccessorGen.paGetStringField(emailAdd_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 3, ((java.lang.String) nullField));
                }
				//passwd
                nullField = paccessorGen.paGetStringField(passwd_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 4, ((java.lang.String) nullField));
                }
				//username
                nullField = paccessorGen.paGetStringField(username_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 5, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 5, ((java.lang.String) nullField));
                }
				//role
                nullField = paccessorGen.paGetStringField(role_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 6, ((java.lang.String) nullField));
                }
				//phoneNum
                nullField = paccessorGen.paGetStringField(phoneNum_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 7, ((java.lang.String) nullField));
                }
				//olstore_address_JPK_
                nullField = addressPnc.encodeOint(paccessorGen.paGetRefField(address_NAME, conn));
                if (nullField == null) {
                    adapter.setNull(pstmt, 8, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 8, ((java.lang.Integer) nullField));
                }
            }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
    private String deleteolstore_userTable(
    	Connection connection,
    	Object conn,
    	PAccessorGen paccessorGen,
    	Olstore_userAccessor paccessor,
    	PName pname) throws PException, SQLException, RdbAdapterException {
		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            pstmt = connection.prepareStatement(res = "DELETE FROM olstore_user WHERE username = ?");
				//username
                nullField = paccessor.paGetUsername();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }

	/**
	 * Is the RdbAdapter to use for managing the various relational database.
	 * This field must not be null after the mapping of the class
	 */
	public RdbAdapter adapter;
	/**
	 * Is the select query to use in the Binding.read method
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The read query is built once time
	 * when the class is mapped.
	 */
	public String readQuery = null;

	/**
	 * Is the select query to use in the Mapping.getPNameIterator method
	 * in the case where the prefetching is activated.
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The extent query is built once time
	 * when the class is mapped.
	 */
	public String prefetchedExtentQuery = null;

	/**
	 * Is the select query to use in the Mapping.getPNameIterator method
	 * in the case where the prefetching is NOT activated.
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The extent query is built once time
	 * when the class is mapped.
	 */
	public String extentQuery = null;

	/**
	 * Building of the structure defining the FROM clause with
	 *  JoinedTable instances
	 */
	private JoinedTable getJoinedTable() {
		JoinedTable jt = new JoinedTable("olstore_user");
		JoinedTable.Join jtj = null;
        return jt;
	}

    public String[] getPrefetchSelectCols() {
		return new String[]{"olstore_user.emailAdd", "olstore_user.fname", "olstore_user.lname", "olstore_user.olstore_address_JPK_", "olstore_user.passwd", "olstore_user.phoneNum", "olstore_user.role", "olstore_user.username"};
	}

	public String getPrefetchWhereClause() {
		return null;
	}

	public java.util.Properties getCNId2Column() {
		java.util.Properties p = new java.util.Properties();
		p.setProperty("", "olstore_user.username");
		return p;
	}

	public List getPrefetchTables() {
		return Collections.singletonList(getJoinedTable());
	}

	/**
	 * Compute the read query with the help of the RdbAdapter
	 */
	public void computeReadQuery() {
		readQuery = adapter.getQuery("olstore_user.emailAdd, olstore_user.fname, olstore_user.lname, olstore_user.olstore_address_JPK_, olstore_user.passwd, olstore_user.phoneNum, olstore_user.role, olstore_user.username",
			Collections.singletonList(getJoinedTable()),
			"olstore_user.username = ?",
			false, false);
	}

	/**
	 * Compute the extent query (without prefetch) with the help of the
	 * RdbAdapter
	 */
	public void computeExtentQuery() throws ExpressionException {
		if (extentQuery == null) {
			extentQuery = adapter.getQuery("olstore_user.username",
				Collections.singletonList(getJoinedTable()),
			null,
			false, false);
		}
	}

	/**
	 * Compute the extent query (with prefetch) with the help of the RdbAdapter
	 */
	public void computePrefetchedExtentQuery() {
		if (prefetchedExtentQuery == null) {
			prefetchedExtentQuery = adapter.getQuery("olstore_user.emailAdd, olstore_user.fname, olstore_user.lname, olstore_user.olstore_address_JPK_, olstore_user.passwd, olstore_user.phoneNum, olstore_user.role, olstore_user.username, olstore_user.username" + adapter.getColumnAliasExpr("pk0") + "",
			Collections.singletonList(getJoinedTable()),
			null,
			false, false);
		}

	}


	public PName decode(ResultSet rs, int startindex, Object png) throws IOException, SQLException, PException {
        return classPNC.decodeString(adapter.getString(rs, startindex, null));
	}



	
	// IMPLEMENTATION OF METHODS FROM THE Loggable INTERFACE

	public void setLogger(Logger l) {
		logger = l;
		((Loggable) friendsGCM).setLogger(l);
		((Loggable) jonasCMRFriend_UserGCM).setLogger(l);
		((Loggable) ordersGCM).setLogger(l);
		((Loggable) purchasedGCM).setLogger(l);
	}

	public void setLoggerFactory(LoggerFactory lf) {
		loggerFactory = lf;
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}
}




class Olstore_userResultIterator implements PNameIterator {
    ResultSet rs;
	private PBinder binder;
	private Olstore_userMapping pclassMapping;
    private boolean hasNext;
    private boolean isForw;
	private PreparedStatement ps;
	private Logger logger;

    private boolean prefetching;
    
    private PPrefetchTupleCollection tc = null;
    private int pnameIndex;

	Olstore_userResultIterator(Object txctx,
		PreparedStatement ps,
		ResultSet r,
		PBinder binder,
		boolean prefetching,
		boolean withSubType,
		Logger l,
		int pi,
		Olstore_userMapping pcm) throws SQLException, MedorException {

		this.ps = ps;
		this.binder = binder;
		this.prefetching = prefetching;
        rs = r;
        logger = l;
        pclassMapping = pcm;
        if (!prefetching) {
        	pnameIndex = 1;
        } else {
        	pnameIndex = pi;
        	tc = new RdbTupleCollection(txctx, r, ps, pnameIndex, logger, pclassMapping, withSubType, this);
        }
        hasNext = (!prefetching) ? rs.next() : tc.next();
        isForw = true;
    }

	public void finalize() throws Throwable {
		if (rs != null)
			rs.close();
		if (ps != null)
			ps.close();
		super.finalize();
	}

    public void close() {
        if (!prefetching) {
            if (rs!= null) {
                try {
                    rs.close();
                } catch(Exception e) {
                }
                rs = null;
            }
            hasNext = false;
            isForw = false;
        }
    }

    public boolean hasNext() {
    	boolean res;
        if (isForw) {
            res = hasNext;
        } else {
            isForw = true;
            try {
                hasNext = (!prefetching) ? rs.next() : tc.next();
                res = hasNext;
            } catch(Exception e) {
                e.printStackTrace();
                res = false;
            }
        }
        if (res == false) {
        	try {
				if (rs != null) {
					rs.close();
					rs = null;
				}
				if (ps != null) {
					ps.close();
					ps = null;
				}
			} catch (SQLException e) {
				rs = null;
				ps = null;
			}
        }
        return res;
    }

    public void remove() throws UnsupportedOperationException, IllegalStateException {
        throw new UnsupportedOperationException();
    }

    public Object next() throws NoSuchElementException {
        try {
            if (!isForw) {
                hasNext = (!prefetching) ? rs.next() : tc.next();
                isForw = true;
            }
            if (!hasNext) {
                throw new NoSuchElementException();
            }
            isForw = false;
            if (prefetching) {
            	tc.getPrefetchBuffer().addPrefetchTuple();
            }
            return pclassMapping.decode(rs, pnameIndex, null);
        } catch (NoSuchElementException e) {
        	throw e;
        } catch (Exception e) {
        	if (logger != null) {
        		logger.log(BasicLevel.ERROR, "Impossible to fetch the next element", e);
        	}
            throw new NoSuchElementException(
                    "Impossible to decode to an ID PName: " + e.getMessage());
        }
    }
}












